#ifndef PV_TYPES_H
#define PV_TYPES_H

#include <libpv_config.h>

#include <stdlib.h>	/* loff_t */
#include <pthread.h>

#include <ptxlist.h>

#include <libpv.h>

#define LEVEL_ALL	7
#define LEVEL_DEBUG	6
#define LEVEL_INFO	5
#define LEVEL_NOTICE	4
#define LEVEL_WARN	3
#define LEVEL_ERROR	2
#define LEVEL_CRIT	1

#ifdef DEBUG
#define dprintf(_level, _fmt, _arg...)			\
do {							\
	if (_libpv_debuglevel >= _level) {		\
		fprintf(stdout, _fmt, ##_arg);		\
		fflush(stdout);				\
	}						\
} while (0)
#else
#define dprintf(_level, _fmt, _arg...)			\
do {							\
	if (_level == LEVEL_CRIT ||			\
	    _level == LEVEL_ERROR ||			\
	    _level == LEVEL_WARN)			\
		fprintf(stderr, _fmt, ##_arg);		\
	} while (0)
#endif

extern int _libpv_debuglevel;

#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif


enum pv_type {
	PV_TYPE_UNDEF = 0,

	PV_TYPE_INT8,
	PV_TYPE_UINT8,

	PV_TYPE_INT16,
	PV_TYPE_UINT16,

	PV_TYPE_INT32,
	PV_TYPE_UINT32,

	PV_TYPE_INT64,
	PV_TYPE_UINT64,

	PV_TYPE_STRING,
};


struct pv_backing_store;
typedef struct pv_backing_store pv_backing_store_t;


struct pv_backing_store {
	int	(*init)		(pv_backing_store_t	*,
				 const char		*filename,
				 size_t			size);

	int	(*exit)		(pv_backing_store_t	*);

	int	(*read)		(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	(*write)	(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	shmid;

	void	*map;		/*
				 * set to NULL for not mmappable backing stores, 
				 * the user will be prompted for a shm segment
				 */
	size_t	size;

	void	*priv;
};


union pv_dflt {
	int8_t		s8;
	uint8_t		u8;

	int16_t		s16;
	uint16_t	u16;

	int32_t		s32;
	uint32_t	u32;

	int64_t		s64;
	uint64_t	u64;
	
	char		*string;
};


union pv_data {
	int8_t		s8[0];
	uint8_t		u8[0];

	int16_t		s16[0];
	uint16_t	u16[0];

	int32_t		s32[0];
	uint32_t	u32[0];

	int64_t		s64[0];
	uint64_t	u64[0];
	
	char		string[0];

	void		*ptr[0];
};


struct pv_engine {
	int			pv_store_size;
	int			pv_sem;		/* SysV semaphore                   */
	int			pv_sem_key;

	pv_backing_store_t	store;
	
	/* PV Data */
	ptxlist			*pv_list;	/* list of process variables        */

	/* JVisu */
	struct jv_server	*jv_server;
};


struct pv {
	char 			*id;		/* id of var */
	char			*name;		/* name of var "description" */
						/* FIXME: make this a language index array... (rsc) */ 

	enum pv_type 		type;
	off_t			offset;
	union pv_dflt		dflt;

	size_t			size;
};

#endif	/* !PV_TYPES_H */
