#include <libpv_config.h>

#include <errno.h>

#include <pv_destroy.h>
#include <pv_lock.h>
#include <pv_bs.h>
#include <pv_jvisu.h>

int pv_destroy_free_pvs(ptxlist	*pv_list)
{
	int finished = 0; 

	if (!pv_list)
		return -EINVAL;

	for (goto_head(&pv_list); !finished; goto_next(&pv_list)) {
			
		struct pv *pv = (struct pv *)list_get_element_data(pv_list);
			
		if (is_last(pv_list))
			finished=1;

		if (pv == NULL) {
			dprintf(LEVEL_ERROR, "(%s) error: trying to free nonexisting list element\n",
				__FUNCTION__);
			continue;
		}

		free(pv->id);
		free(pv->name);
		if (pv->type == PV_TYPE_STRING)
			free(pv->dflt.string);

		pv_list = list_del_element(pv_list);
	}

	return 0;
}


int pv_destroy (pv_engine_t	*engine_anon)
{
	struct pv_engine	*engine = (struct pv_engine *)engine_anon;
	int			err = 0, ret;

	if (!engine_anon)
		return -EINVAL;

	/* shutdown jvisu socket server */
	dprintf(LEVEL_INFO, "(%s) stopping server\n", __FUNCTION__);

	ret = pv_jvisu_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy the jvisu server stuff, err %d\n",
			__PRETTY_FUNCTION__, err);
	}

	ret = pv_lock_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy lock, err %d\n",
			__FUNCTION__, err);
	}

	ret = pv_bs_destroy(engine);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy backing store, err %d\n",
			__FUNCTION__, err);
	}
	

	/* remove process variables */
	ret = pv_destroy_free_pvs(engine->pv_list);
	if (ret) {
		err = ret;
		dprintf(LEVEL_ERROR, "(%s) failed to destroy pv_list, err %d\n",
			__FUNCTION__, err);
	}
	
	return err;
}
