#include <libpv_config.h>

#include <errno.h>
#include <inttypes.h>
#include <string.h>

#include <pv_jvisu.h>
#include <jv_server.h>


int pv_jvisu_init(struct pv_engine	*engine,
		  uint16_t		port)
{
	struct jv_server	*jv_server;
	int			err, sd;

	if (port == 0)
		return 0;

	if (engine->jv_server)
		return -EBUSY;

	jv_server = engine->jv_server = (struct jv_server *)calloc(1, sizeof(struct jv_server));
	if (!engine->jv_server) {
		dprintf(LEVEL_CRIT, "(%s) failed to allocate memory for jv_server\n",
			__PRETTY_FUNCTION__);
		err = -ENOMEM;
		goto exit_0;
	}

	jv_server->pv_engine = engine;

	/* open bind and listen on socket */
	sd = jv_server_start_socket(jv_server, port);
	if (sd < 0) {
		dprintf(LEVEL_CRIT, "(%s) cannot open socket (port: %d)\n",
			__PRETTY_FUNCTION__, port);
		err = -errno;
		goto exit_1;
	}
	dprintf(LEVEL_DEBUG, "(%s) active (sd=%d), waiting for connections on port %d\n",
		__PRETTY_FUNCTION__, sd, port);
	
	/* preset active SDs with -1 */
	memset(&jv_server->jv_active_sock, 0xff, sizeof (jv_server->jv_active_sock));

	/* store active socket */
	jv_server->jv_active_sock[0].sd = sd;
	
	FD_ZERO(&jv_server->jv_read_fdset);
	FD_ZERO(&jv_server->jv_write_fdset);
	
	/* server is always set */
	FD_SET(sd, &jv_server->jv_read_fdset);

	err = pthread_create(&jv_server->jvsocket_thread, 0, jv_server_socket_task, jv_server);
	if (err) {
		dprintf(LEVEL_CRIT, "(%s) failed to create jvisu thread (err %d)\n",
			__PRETTY_FUNCTION__, err);
		goto exit_1;
	}

	return 0;

 exit_1:
	free(engine->jv_server);
 exit_0:
	return err;
}


int pv_jvisu_destroy(struct pv_engine	*engine)
{
	struct jv_server	*jv_server = engine->jv_server;

	if (!jv_server)		/* no mem alloced, thus not running */
		return 0;

	jv_server->jvisu_stop = 1;

	dprintf(LEVEL_DEBUG, "(%s) waiting for server thread\n", __FUNCTION__);
	pthread_join(jv_server->jvsocket_thread, NULL);
	dprintf(LEVEL_DEBUG, "(%s) joined jvisu server thread\n", __FUNCTION__);

	return 0;
}
